@@ -1,5 +1,5 @@
 """
-Copyright (c) 2020 Cypress Semiconductor Corporation
+Copyright (c) 2020-2021 Cypress Semiconductor Corporation
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -19,4 +19,5 @@ logger = logging.getLogger(__name__)
 
 
 def update_progress(progress):
-    logger.info('\r[ {0}{1} ] {2}%'.format('#' * progress, ' ' * int(100 - progress), progress))
+    logging.info('\r[ %s%s ] %d%%', '#' * progress, ' ' * int(100 - progress),
+                 progress)
